package core

import (
	"encoding/json"
	"fmt"
	"github.com/louisevanderlith/husk/collections"
	"log"
	"net/http"
	"reflect"
	"strings"
)

func ScrapeCars() (collections.Enumerable, error) {
	carsUrl := "https://www.cars.co.za/sell-car-process/"

	minYear := 1950
	maxYear := 2021

	var result []Vehicle

	for i := minYear; i <= maxYear; i++ {
		items, err := loadManufacturers(carsUrl, i)

		if err != nil {
			return nil, err
		}

		result = append(result, items...)
	}

	return collections.ReadOnlyList(reflect.ValueOf(result)), nil
}

func loadManufacturers(bUrl string, year int) ([]Vehicle, error) {
	url := fmt.Sprintf("%s?sellCarYear=%v", bUrl, year)

	resp, err := http.Get(url)

	if err != nil {
		return nil, err
	}

	manus := make(map[string]string)
	dec := json.NewDecoder(resp.Body)
	err = dec.Decode(manus)

	if err != nil {
		return nil, err
	}

	var result []Vehicle
	for _, v := range manus {
		cars, err := loadModels(bUrl, year, v)

		if err != nil {
			return nil, err
		}

		result = append(result, cars...)
	}

	return result, nil
}

func loadModels(bUrl string, year int, manufacturer string) ([]Vehicle, error) {
	url := fmt.Sprintf("%s?sellCarYear=%v&sellCarMake=%s", bUrl, year, make)

	resp, err := http.Get(url)

	if err != nil {
		log.Println(err)
	}

	models := make(map[string]string)
	dec := json.NewDecoder(resp.Body)
	err = dec.Decode(models)

	if err != nil {
		return nil, err
	}

	var result []Vehicle
	for _, v := range models {
		cars, err := loadTrim(bUrl, year, manufacturer, v)

		if err != nil {
			return nil, err
		}

		result = append(result, cars...)
	}

	return result, nil
}

func loadTrim(bUrl string, year int, manufacturer string, model string) ([]Vehicle, error) {
	url := fmt.Sprintf("%s?sellCarYear=%v&sellCarMake=%s&sellCarModel=%s", bUrl, year, make, model)

	resp, err := http.Get(url)

	if err != nil {
		log.Println(err)
	}

	trims := make(map[string]string)
	dec := json.NewDecoder(resp.Body)
	err = dec.Decode(trims)

	if err != nil {
		log.Println(err)
	}

	var result []Vehicle

	for v := range trims {
		rec, err := createVehicle(year, manufacturer, model, v)

		if err != nil {
			return nil, err
		}
		result = append(result, rec)
	}

	return result, nil
}

func createVehicle(year int, make, model, variant string) (Vehicle, error) {
	gearType := "Unknown"
	gasType := "Unknown"

	if strings.Contains(variant, "(Manual)") {
		gearType = "Manual"
		variant = strings.Replace(variant, "(Manual)", "", -1)
	} else {
		gearType = "Auto"
		variant = strings.Replace(variant, "(Auto)", "", -1)
	}

	if strings.Contains(variant, "(Petrol)") {
		gasType = "Petrol"
		variant = strings.Replace(variant, "(Petrol)", "", -1)
	} else {
		gasType = "Diesel"
		variant = strings.Replace(variant, "(Diesel)", "", -1)
	}

	obj := Vehicle{
		VINKey:  nil,
		FullVIN: "",
		Series: SeriesInfo{
			Model:         model,
			Manufacturer:  make,
			AssemblyPlant: "Unknown",
			Month:         1,
			Year:          year,
			Trim:          variant,
		},
		Colour:  "Unknown",
		PaintNo: "P0",
		Engine: Engine{
			Code:     "",
			SerialNo: "",
			Output:   0,
			Fuel:     gasType,
		},
		Gearbox: Gearbox{
			Code:     "",
			SerialNo: "",
			Gears:    0,
			Type:     gearType,
		},
		BodyStyle: 0,
		Doors:     0,
		Extra:     nil,
		Spare:     false,
		Service:   false,
		Condition: "",
		Issues:    "",
		Mileage:   0,
	}

	err := obj.Valid()
	if err != nil {
		return Vehicle{}, err
	}

	return obj, nil
}
